@@ -9,13 +9,12 @@ const PopupMenu = imports.ui.popupMenu;
 const Applet = imports.ui.applet;
 const SignalManager = imports.misc.signalManager;
 
-const {each, findIndex, tryFn, unref, trySpawnCommandLine, spawn_async} = imports.misc.util;
+const {each, findIndex, tryFn, unref, trySpawnCommandLine, spawn_async, getDesktopActionIcon} = imports.misc.util;
 const {
     CLOSE_BTN_SIZE,
     CLOSED_BUTTON_STYLE,
     OPACITY_OPAQUE,
     RESERVE_KEYS,
-    ICON_NAMES,
     FavType,
     autoStartStrDir
 } = require('./constants');
@@ -107,7 +106,7 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
                 }
             }
             // Workspace
-            if ((length = global.screen.n_workspaces) > 1) {
+            if ((length = global.workspace_manager.n_workspaces) > 1) {
                 if (this.groupState.lastFocused && this.groupState.lastFocused.is_on_all_workspaces()) {
                     item = createMenuItem({label: _('Only on this workspace')});
                     this.signals.connect(item, 'activate', () => {
@@ -131,7 +130,7 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
 
                     let connectWorkspaceEvent = (ws, j) => {
                         this.signals.connect(ws, 'activate', () => {
-                            this.groupState.lastFocused.change_workspace(global.screen.get_workspace_by_index(j));
+                            this.groupState.lastFocused.change_workspace(global.workspace_manager.get_workspace_by_index(j));
                         });
                     };
                     for (let i = 0; i < length; i++) {
@@ -166,9 +165,8 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
         subMenu.menu.addMenuItem(item);
 
         item = createMenuItem({label: _("Remove '%s'").format(_("Grouped window list")), icon: 'edit-delete'});
-        this.signals.connect(item, 'activate', () => {
-            AppletManager._removeAppletFromPanel(this.state.uuid, this.state.instance_id);
-        });
+        this.signals.connect(item, 'activate', (actor, event) => this.state.trigger('removeApplet', event));
+
         subMenu.menu.addMenuItem(item);
         this.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
 
@@ -181,7 +179,7 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
                 let defualtPlaces = this.listDefaultPlaces();
                 let bookmarks = this.listBookmarks();
                 let devices = this.listDevices();
-                let places = defualtPlaces.concat(bookmarks).concat(devices);
+                let places = [...defualtPlaces, ...bookmarks, ...devices];
                 let handlePlaceLaunch = (item, i) => {
                     this.signals.connect(item, 'activate', () => places[i].launch());
                 };
@@ -227,6 +225,15 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
             }
         }
 
+        if (Main.gpu_offload_supported && !hasWindows) {
+            item = createMenuItem({label: _("Run with NVIDIA GPU"), icon: 'cpu'});
+
+            this.signals.connect(item, 'activate', () => this.groupState.trigger('launchNewInstance', true));
+
+            this.addMenuItem(item);
+            this.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
+        }
+
         // Actions
         tryFn(() => {
             if (!this.groupState.appInfo) return;
@@ -234,21 +241,9 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
             if (actions) {
                 this.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
                 let handleAction = (action) => {
-                    let actionID = '';
-                    if (action.toUpperCase() === action) {
-                        actionID = action.toLowerCase();
-                    } else {
-                        // first letter lowercase, replace uppercase with _+lowercase
-                        actionID = action.charAt(0).toLowerCase() + action.slice(1);
-                        actionID = actionID.replace(/([A-Z])/g, '_$1').toLowerCase();
-                    }
-                    actionID = actionID.replace(/-/g, '_');
-
-                    let icon = 'application-x-executable';
-
-                    if (ICON_NAMES.hasOwnProperty(actionID)) {
-                        icon = ICON_NAMES[actionID];
-                    }
+                    let icon = getDesktopActionIcon(action);
+                    if (icon == null)
+                        icon = 'application-x-executable';
 
                     item = createMenuItem({
                         label: _(this.groupState.appInfo.get_action_name(action)),
@@ -266,14 +261,6 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
                 }
                 this.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
             }
-            if (this.state.settings.launchNewInstance && (!actions || actions.length === 0) && !isWindowBacked) {
-                item = createMenuItem({label: _('New Window'), icon: 'window-new'});
-                this.signals.connect(item, 'activate', () => this.groupState.trigger('launchNewInstance'));
-                this.addMenuItem(item);
-                if (!actions || actions.length === 0) {
-                    this.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
-                }
-            }
         }, () => {
             if (isWindowBacked) {
                 this.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
@@ -344,6 +331,7 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
                 });
             }
             this.addMenuItem(item);
+            item.setSensitive(this.groupState.lastFocused.can_maximize());
 
             if (this.groupState.metaWindows && this.groupState.metaWindows.length > 1) {
                 // Close others
@@ -382,11 +370,15 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
 
     onToggled(actor, isOpening) {
         this.state.set({menuOpen: this.isOpen});
+    }
 
-        if (!isOpening) return;
+    toggle() {
+        if (!this.isOpen) {
+            this.removeAll();
+            this.populateMenu();
+        }
 
-        this.removeAll();
-        this.populateMenu();
+        Applet.AppletPopupMenu.prototype.toggle.call(this);
     }
 
     toggleAutostart() {
@@ -519,9 +511,7 @@ class WindowThumbnail {
         this.state = params.state;
         this.stateConnectId = this.state.connect({
             scrollActive: () => {
-                if (this.state.overlayPreview) {
-                    this.destroyOverlayPreview();
-                }
+                this.destroyOverlayPreview();
             },
             thumbnailCloseButtonOffset: ({thumbnailCloseButtonOffset}) => {
                 this.button.style = CLOSED_BUTTON_STYLE + `position: ${thumbnailCloseButtonOffset}px -2px;`;
@@ -628,9 +618,7 @@ class WindowThumbnail {
             && (!this.lastEnterActor
                 || (this.lastEnterActor.indexOf('StButton') === -1)
                     && this.lastEnterActor.indexOf('ClutterClone') === -1)) {
-            if (this.state.overlayPreview) {
-                this.destroyOverlayPreview();
-            }
+            this.destroyOverlayPreview();
             this.hoverPeek(this.state.settings.peekOpacity);
         }
         this.lastEnterActor = actorString;
@@ -679,7 +667,6 @@ class WindowThumbnail {
     handleCloseClick() {
         this.onLeave();
         this.stopClick = true;
-        this.hoverPeek(OPACITY_OPAQUE);
 
         this.metaWindow.delete(global.get_current_time());
         if (!this.groupState.metaWindows || this.groupState.metaWindows.length <= 1) {
@@ -721,16 +708,16 @@ class WindowThumbnail {
         }
         // Create our own thumbnail if it doesn't exist
         if (this.metaWindowActor) {
-            this.signals.disconnect('size-changed', this.metaWindowActor);
+            this.signals.disconnect('notify::size', this.metaWindowActor);
         } else {
             this.metaWindowActor = this.metaWindow.get_compositor_private();
         }
         if (this.metaWindowActor && !this.metaWindowActor.is_finalized()) {
-            this.signals.connect(this.metaWindowActor, 'size-changed', () => this.refreshThumbnail());
+            this.signals.connect(this.metaWindowActor, 'notify::size', () => this.refreshThumbnail());
 
             let windowTexture = this.metaWindowActor.get_texture();
             if (!windowTexture) return;
-            let [width, height] = windowTexture.get_size();
+            let [width, height] = this.metaWindowActor.get_size();
             let scale = Math.min(1.0, thumbnailWidth / width, thumbnailHeight / height) * global.ui_scale;
             width = Math.round(width * scale);
             height = Math.round(height * scale);
@@ -741,8 +728,8 @@ class WindowThumbnail {
                 this.thumbnailActor.child.width = width;
                 this.thumbnailActor.child.height = height;
             } else {
-                this.thumbnailActor.child = new Clutter.Clone({
-                    source: windowTexture,
+                this.thumbnailActor.child = new Clutter.Actor({
+                    content: windowTexture,
                     reactive: true,
                     width,
                     height
@@ -827,7 +814,6 @@ class WindowThumbnail {
 
     hoverPeek(opacity) {
         if (!this.state.settings.enablePeek
-            || this.state.overlayPreview
             || this.state.scrollActive
             || (this.metaWindowActor && this.metaWindowActor.is_finalized())) {
             return;
@@ -836,37 +822,42 @@ class WindowThumbnail {
             this.metaWindowActor = this.metaWindow.get_compositor_private();
         }
         this.state.set({
-            overlayPreview: new Clutter.Clone({
-                source: this.metaWindowActor.get_texture(),
+            lastOverlayPreview: new Clutter.Actor({
+                content: this.metaWindowActor.get_texture(),
                 opacity: 0
             })
         });
+
         let [x, y] = this.metaWindowActor.get_position();
-        this.state.overlayPreview.set_position(x, y);
-        global.overlay_group.add_child(this.state.overlayPreview);
-        global.overlay_group.set_child_above_sibling(this.state.overlayPreview, null);
-        setOpacity(this.state.settings.peekTimeIn, this.state.overlayPreview, opacity);
+        this.state.lastOverlayPreview.set_position(x, y);
+        global.overlay_group.add_child(this.state.lastOverlayPreview);
+        global.overlay_group.set_child_above_sibling(this.state.lastOverlayPreview, null);
+        setOpacity(this.state.settings.peekTimeIn, this.state.lastOverlayPreview, opacity);
     }
 
     destroyOverlayPreview() {
-        if (!this.state.overlayPreview) return;
+        if (!this.state.lastOverlayPreview) return;
 
         if (this.state.settings.peekTimeOut) {
+            let currOverlayPreview = this.state.lastOverlayPreview;
             setOpacity(
                 this.state.settings.peekTimeOut,
-                this.state.overlayPreview,
+                currOverlayPreview,
                 0,
-                () => this._destroyOverlayPreview()
+                () => this._destroyOverlayPreview(currOverlayPreview)
             );
         } else {
-            this._destroyOverlayPreview();
+            this._destroyOverlayPreview(this.state.lastOverlayPreview);
         }
     }
 
-    _destroyOverlayPreview() {
-        global.overlay_group.remove_child(this.state.overlayPreview);
-        this.state.overlayPreview.destroy();
-        this.state.set({overlayPreview: null});
+    _destroyOverlayPreview(overlayPreview) {
+        global.overlay_group.remove_child(overlayPreview);
+        overlayPreview.destroy();
+
+        if(overlayPreview === this.state.lastOverlayPreview) {
+            this.state.set({lastOverlayPreview: null});
+        }
     }
 
     destroy() {
@@ -888,6 +879,8 @@ class AppThumbnailHoverMenu extends PopupMenu.PopupMenu {
         super._init.call(this, groupState.trigger('getActor'), state.orientation, 0.5);
         this.state = state;
         this.groupState = groupState;
+        this.shouldClose = true;
+        this.isOpen = false;
         this.setCustomStyleClass("grouped-window-list-thumbnail-menu");
 
         this.connectId = this.groupState.connect({
@@ -912,11 +905,10 @@ class AppThumbnailHoverMenu extends PopupMenu.PopupMenu {
             },
             addThumbnailToMenu: (win) => {
                 if (this.isOpen) {
-                    this.close(true);
                     this.addThumbnail(win);
-                    this.open(true);
                     return;
                 }
+
                 this.queuedWindows.push(win);
             },
             removeThumbnailFromMenu: (win) => {
@@ -1123,7 +1115,7 @@ class AppThumbnailHoverMenu extends PopupMenu.PopupMenu {
             } else {
                 index = this.appThumbnails.length - 1;
             }
-        } else if (symbol === Clutter.KEY_Return && entered) {
+        } else if ((symbol === Clutter.KEY_Return || symbol === Clutter.KEY_KP_Enter) && entered) {
             this.appThumbnails[i].connectToWindow(null, 1);
         } else if (symbol === closeArg) {
             this.appThumbnails[i].onLeave();
