@@ -1,9 +1,12 @@
-#!/usr/bin/env python2
+#!/usr/bin/python3
 
+import subprocess
 import gi
 gi.require_version("Gtk", "3.0")
-from gi.repository import Gtk, CDesktopEnums
-from GSettingsWidgets import *
+
+from gi.repository import Gtk
+from SettingsWidgets import SidePage, GSettingsDependencySwitch, DependencyCheckInstallButton, GSettingsSoundFileChooser
+from xapp.GSettingsWidgets import *
 
 DPI_FACTOR_LARGE         = 1.25
 DPI_FACTOR_NORMAL        = 1.0
@@ -11,8 +14,11 @@ DPI_FACTOR_NORMAL        = 1.0
 HIGH_CONTRAST_THEME      = "HighContrast"
 KEY_TEXT_SCALING_FACTOR  = "text-scaling-factor"
 KEY_GTK_THEME            = "gtk-theme"
+KEY_GTK_THEME_BACKUP     = "gtk-theme-backup"
 KEY_ICON_THEME           = "icon-theme"
+KEY_ICON_THEME_BACKUP    = "icon-theme-backup"
 KEY_WM_THEME             = "theme"
+KEY_WM_THEME_BACKUP      = "theme-backup"
 
 
 class Module:
@@ -27,7 +33,7 @@ class Module:
 
     def on_module_selected(self):
         if not self.loaded:
-            print "Loading Accessibility module"
+            print("Loading Accessibility module")
 
             self.iface_settings = Gio.Settings(schema_id="org.cinnamon.desktop.interface")
             self.wm_settings = Gio.Settings(schema_id="org.cinnamon.desktop.wm.preferences");
@@ -62,13 +68,19 @@ class Module:
             settings.add_row(switch)
 
             switch = GSettingsDependencySwitch(_("Screen reader"),
-                                     "org.cinnamon.desktop.a11y.applications",
-                                     "screen-reader-enabled",
-                                     None,
-                                     ["orca"],
-                                     ["gnome-orca"])
+                                               "org.cinnamon.desktop.a11y.applications",
+                                               "screen-reader-enabled",
+                                               None,
+                                               ["orca"],
+                                               ["gnome-orca"])
             settings.add_row(switch)
 
+            def config_orca(button):
+                subprocess.Popen(['orca', '--setup'])
+
+            button = Button(_("Configure screen reader"), config_orca)
+            settings.add_reveal_row(button, "org.cinnamon.desktop.a11y.applications", "screen-reader-enabled")
+
 # Desktop Zoom
 
             settings = page.add_section(_("Desktop Zoom"))
@@ -79,7 +91,7 @@ class Module:
             spin = GSettingsSpinButton(_("Magnification"), "org.cinnamon.desktop.a11y.magnifier", "mag-factor", None, 1.0, 15.0, step=0.5)
             settings.add_reveal_row(spin, "org.cinnamon.desktop.a11y.applications", "screen-magnifier-enabled")
 
-            zoom_key_options = [["", _("Disabled")], ["<Alt>", "<Alt>"],["<Super>", "<Super>"],["<Control>", "<Control>"]]
+            zoom_key_options = [["", _("Disabled")], ["<Alt>", "<Alt>"],["<Super>", "<Super>"],["<Control>", "<Control>"], ["<Shift>", "<Shift>"]]
             widget = GSettingsComboBox(_("Mouse wheel modifier"), "org.cinnamon.desktop.wm.preferences", "mouse-button-zoom-modifier", zoom_key_options)
             widget.set_tooltip_text(_("While this modifier is pressed, mouse scrolling will increase or decrease zoom."))
             settings.add_reveal_row(widget, "org.cinnamon.desktop.a11y.applications", "screen-magnifier-enabled")
@@ -145,13 +157,26 @@ class Module:
 
             settings.add_row(switch)
 
-            keyboard_type_options = [["touch",     _("Touch")],
+            keyboard_type_options = [["tablet",     _("Tablet")],
+                                     ["touch",     _("Touch")],
                                      ["fullscale", _("Full scale")],
                                      ["scan",      _("Scanning")]]
 
             widget = GSettingsComboBox(_("Keyboard layout"), "org.cinnamon.keyboard", "keyboard-type", keyboard_type_options)
             settings.add_reveal_row(widget, "org.cinnamon.desktop.a11y.applications", "screen-keyboard-enabled")
 
+            keyboard_position_options = [["bottom",     _("At the bottom of the screen")],
+                                         ["top",     _("At the top of the screen")]]
+            widget = GSettingsComboBox(_("Keyboard position"), "org.cinnamon.keyboard", "keyboard-position", keyboard_position_options)
+            settings.add_reveal_row(widget, "org.cinnamon.desktop.a11y.applications", "screen-keyboard-enabled")
+
+            keyboard_size_options = [[4,     _("A quarter of the screen")],
+                                     [3,     _("A third of the screen")],
+                                     [2, _("Half of the screen")]]
+
+            widget = GSettingsComboBox(_("Keyboard size"), "org.cinnamon.keyboard", "keyboard-size", keyboard_size_options)
+            settings.add_reveal_row(widget, "org.cinnamon.desktop.a11y.applications", "screen-keyboard-enabled")
+
             activation_mode_options = [["accessible", _("Show the keyboard any time something expects input")],
                                        ["on-demand",  _("Show keyboard only when the user activates it")]];
 
@@ -273,7 +298,7 @@ class Module:
                                     "org.cinnamon.desktop.a11y.keyboard",
                                     "slowkeys-delay",
                                     _("Short"), _("Long"),
-                                    0, 500, 10)
+                                    0, 500, 10, show_value=False)
 
             settings.add_reveal_row(slider, "org.cinnamon.desktop.a11y.keyboard", "slowkeys-enable")
 
@@ -299,7 +324,7 @@ class Module:
                                     "org.cinnamon.desktop.a11y.keyboard",
                                     "bouncekeys-delay",
                                     _("Short"), _("Long"),
-                                    0, 900, 10)
+                                    0, 900, 10, show_value=False)
 
             settings.add_reveal_row(slider, "org.cinnamon.desktop.a11y.keyboard", "bouncekeys-enable")
 
@@ -323,7 +348,7 @@ class Module:
                                     "org.cinnamon.desktop.a11y.keyboard",
                                     "mousekeys-init-delay",
                                     _("Shorter"), _("Longer"),
-                                    10, 2000, 10)
+                                    10, 2000, 10, show_value=False)
 
             settings.add_reveal_row(slider, "org.cinnamon.desktop.a11y.keyboard", "mousekeys-enable")
 
@@ -331,7 +356,7 @@ class Module:
                                     "org.cinnamon.desktop.a11y.keyboard",
                                     "mousekeys-accel-time",
                                     _("Shorter"), _("Longer"),
-                                    10, 2000, 10)
+                                    10, 2000, 10, show_value=False)
 
             settings.add_reveal_row(slider, "org.cinnamon.desktop.a11y.keyboard", "mousekeys-enable")
 
@@ -339,33 +364,13 @@ class Module:
                                     "org.cinnamon.desktop.a11y.keyboard",
                                     "mousekeys-max-speed",
                                     _("Slower"), _("Faster"),
-                                    1, 500, 1)
+                                    1, 500, 1, show_value=False)
 
             settings.add_reveal_row(slider, "org.cinnamon.desktop.a11y.keyboard", "mousekeys-enable")
 
-# Dependency Checker
-
-            settings = page.add_reveal_section(_("Simulated secondary click and hover click"))
-            self.dc_section = settings
-
-            install_widget = SettingsWidget()
-
-            self.dep_button = DependencyCheckInstallButton(_("Checking dependencies"),
-                                                           _("Please install: %s") % ("mousetweaks"),
-                                                           ["mousetweaks"],
-                                                           Gtk.Alignment(),
-                                                           self.on_dep_satisfied)
-
-            install_widget.pack_start(self.dep_button, True, False, 0)
-
-            settings.add_row(install_widget)
-
-            self.dc_section._revealer.set_reveal_child(True)
-
 # Secondary click
 
-            settings = page.add_reveal_section(_("Simulated secondary click"))
-            self.ssc_section = settings
+            settings = page.add_section(_("Simulated secondary click"))
 
             switch = GSettingsSwitch(_("Trigger a secondary click by holding down the primary button"),
                                      "org.cinnamon.desktop.a11y.mouse",
@@ -378,14 +383,13 @@ class Module:
                                     "org.cinnamon.desktop.a11y.mouse",
                                     "secondary-click-time",
                                     _("Shorter"), _("Longer"),
-                                    0.5, 3.0, 0.1)
+                                    0.5, 3.0, 0.1, show_value=False)
 
             settings.add_reveal_row(slider, "org.cinnamon.desktop.a11y.mouse", "secondary-click-enabled")
 
 # Hover Click
 
-            settings = page.add_reveal_section(_("Hover click"))
-            self.hc_section = settings
+            settings = page.add_section(_("Hover click"))
 
             switch = GSettingsSwitch(_("Trigger a click when the pointer hovers"),
                                      "org.cinnamon.desktop.a11y.mouse",
@@ -398,7 +402,7 @@ class Module:
                                     "org.cinnamon.desktop.a11y.mouse",
                                     "dwell-time",
                                     _("Short"), _("Long"),
-                                    0.2, 3.0, 0.1)
+                                    0.2, 3.0, 0.1, show_value=False)
 
             settings.add_reveal_row(slider, "org.cinnamon.desktop.a11y.mouse", "dwell-click-enabled")
 
@@ -406,15 +410,10 @@ class Module:
                                     "org.cinnamon.desktop.a11y.mouse",
                                     "dwell-threshold",
                                     _("Small"), _("Large"),
-                                    1, 30, 1)
+                                    1, 30, 1, show_value=False)
 
             settings.add_reveal_row(slider, "org.cinnamon.desktop.a11y.mouse", "dwell-click-enabled")
 
-    def on_dep_satisfied(self):
-        self.ssc_section._revealer.set_reveal_child(True)
-        self.hc_section._revealer.set_reveal_child(True)
-        self.dc_section._revealer.destroy()
-
     def zoom_stack_get(self, lens_mode):
         ret = "screen"
 
@@ -434,12 +433,24 @@ class Module:
         if active:
             ret = HIGH_CONTRAST_THEME
 
+            theme = self.iface_settings.get_string(KEY_GTK_THEME)
+            self.iface_settings.set_string(KEY_GTK_THEME_BACKUP, theme)
+
+            theme = self.iface_settings.get_string(KEY_ICON_THEME)
+            self.iface_settings.set_string(KEY_ICON_THEME_BACKUP, theme)
             self.iface_settings.set_string(KEY_ICON_THEME, HIGH_CONTRAST_THEME)
+
+            theme = self.wm_settings.get_string(KEY_WM_THEME)
+            self.wm_settings.set_string(KEY_WM_THEME_BACKUP, theme)
             self.wm_settings.set_string(KEY_WM_THEME, HIGH_CONTRAST_THEME)
         else:
-            ret = self.iface_settings.get_default_value(KEY_GTK_THEME).get_string()
-            self.iface_settings.reset(KEY_ICON_THEME)
-            self.wm_settings.reset(KEY_WM_THEME)
+            ret = self.iface_settings.get_string(KEY_GTK_THEME_BACKUP)
+
+            theme = self.iface_settings.get_string(KEY_ICON_THEME_BACKUP)
+            self.iface_settings.set_string(KEY_ICON_THEME, theme)
+
+            theme = self.wm_settings.get_string(KEY_WM_THEME_BACKUP)
+            self.wm_settings.set_string(KEY_WM_THEME, theme)
 
         return ret
 
@@ -455,5 +466,3 @@ class Module:
             ret = self.iface_settings.get_default_value(KEY_TEXT_SCALING_FACTOR).get_double()
 
         return ret
-
-
