@@ -1,10 +1,13 @@
 #!/usr/bin/python3
 
-from gi.repository.Gtk import SizeGroup, SizeGroupMode
+import os
 
-from GSettingsWidgets import *
+from gi.repository import Gtk
+
+from xapp.GSettingsWidgets import *
 from CinnamonGtkSettings import CssRange, CssOverrideSwitch, GtkSettingsSwitch, PreviewWidget, Gtk2ScrollbarSizeEditor
-from SettingsWidgets import LabelRow
+from SettingsWidgets import LabelRow, SidePage, walk_directories
+from ChooserButtonWidgets import PictureChooserButton
 from ExtensionCore import DownloadSpicesPage
 from Spices import Spice_Harvester
 
@@ -37,6 +40,7 @@ class Module:
         self.window = None
         sidePage = SidePage(_("Themes"), self.icon, self.keywords, content_box, module=self)
         self.sidePage = sidePage
+        self.refreshing = False # flag to ensure we only refresh once at any given moment
 
     def on_module_selected(self):
         if not self.loaded:
@@ -48,23 +52,20 @@ class Module:
             self.sidePage.add_widget(self.sidePage.stack)
 
             self.settings = Gio.Settings.new("org.cinnamon.desktop.interface")
-            self.wm_settings = Gio.Settings.new("org.cinnamon.desktop.wm.preferences")
             self.cinnamon_settings = Gio.Settings.new("org.cinnamon.theme")
 
+            self.scale = self.window.get_scale_factor()
+
             self.icon_chooser = self.create_button_chooser(self.settings, 'icon-theme', 'icons', 'icons', button_picture_size=ICON_SIZE, menu_pictures_size=ICON_SIZE, num_cols=4)
             self.cursor_chooser = self.create_button_chooser(self.settings, 'cursor-theme', 'icons', 'cursors', button_picture_size=32, menu_pictures_size=32, num_cols=4)
             self.theme_chooser = self.create_button_chooser(self.settings, 'gtk-theme', 'themes', 'gtk-3.0', button_picture_size=35, menu_pictures_size=35, num_cols=4)
-            self.metacity_chooser = self.create_button_chooser(self.wm_settings, 'theme', 'themes', 'metacity-1', button_picture_size=32, menu_pictures_size=32, num_cols=4)
-            self.cinnamon_chooser = self.create_button_chooser(self.cinnamon_settings, 'name', 'themes', 'cinnamon', button_picture_size=60, menu_pictures_size=60, num_cols=4)
+            self.cinnamon_chooser = self.create_button_chooser(self.cinnamon_settings, 'name', 'themes', 'cinnamon', button_picture_size=60, menu_pictures_size=60*self.scale, num_cols=4)
 
             page = SettingsPage()
             self.sidePage.stack.add_titled(page, "themes", _("Themes"))
 
             settings = page.add_section(_("Themes"))
 
-            widget = self.make_group(_("Window borders"), self.metacity_chooser)
-            settings.add_row(widget)
-
             widget = self.make_group(_("Icons"), self.icon_chooser)
             settings.add_row(widget)
 
@@ -91,9 +92,18 @@ class Module:
             widget = GSettingsSwitch(_("Show icons on buttons"), "org.cinnamon.settings-daemon.plugins.xsettings", "buttons-have-icons")
             settings.add_row(widget)
 
+            try:
+                import tinycss2
+            except:
+                self.refresh()
+                return
+
             settings = page.add_section(_("Scrollbar behavior"))
 
-            switch = GtkSettingsSwitch(_("Warp sliders when clicking in a trough"), "gtk-primary-button-warps-slider")
+            # Translators: The 'trough' is the part of the scrollbar that the 'handle'
+            # rides in.  This setting determines whether clicking in that trough somewhere
+            # jumps directly to the new position, or if it only scrolls towards it.
+            switch = GtkSettingsSwitch(_("Jump to position when clicking in a trough"), "gtk-primary-button-warps-slider")
             settings.add_row(switch)
 
             widget = GSettingsSwitch(_("Use overlay scroll bars"), "org.cinnamon.desktop.interface", "gtk-overlay-scrollbars")
@@ -105,10 +115,17 @@ class Module:
             settings.add_row(switch)
             self.scrollbar_switch = switch.content_widget
 
-            widget = CssRange(_("Scrollbar width"), "scrollbar slider", ["min-width", "min-height"], 5, 100, "px", None, switch)
+            widget = CssRange(_("Scrollbar width"), "scrollbar slider", ["min-width", "min-height"], 2, 40, "px", None, switch)
             settings.add_reveal_row(widget)
-            widget.sync_initial_switch_state()
+
+            try:
+                widget.sync_initial_switch_state()
+            except PermissionError as e:
+                print(e)
+                switch.set_sensitive(False)
+
             self.scrollbar_css_range = widget.content_widget
+            self.scrollbar_css_range.get_adjustment().set_page_increment(2.0)
 
             switch.content_widget.connect("notify::active", self.on_css_override_active_changed)
             widget.content_widget.connect("value-changed", self.on_range_slider_value_changed)
@@ -155,13 +172,16 @@ class Module:
             self.gtk2_scrollbar_editor.set_size(widget.get_value())
 
     def on_file_changed(self, file, other, event, data):
-        self.refresh()
+        if self.refreshing:
+            return
+        self.refreshing = True
+        GLib.timeout_add_seconds(5, self.refresh)
 
     def refresh(self):
         choosers = []
         choosers.append((self.cursor_chooser, "cursors", self._load_cursor_themes(), self._on_cursor_theme_selected))
         choosers.append((self.theme_chooser, "gtk-3.0", self._load_gtk_themes(), self._on_gtk_theme_selected))
-        choosers.append((self.metacity_chooser, "metacity-1", self._load_metacity_themes(), self._on_metacity_theme_selected))
+        # choosers.append((self.metacity_chooser, "metacity-1", self._load_metacity_themes(), self._on_metacity_theme_selected))
         choosers.append((self.cinnamon_chooser, "cinnamon", self._load_cinnamon_themes(), self._on_cinnamon_theme_selected))
         choosers.append((self.icon_chooser, "icons", self._load_icon_themes(), self._on_icon_theme_selected))
         for chooser in choosers:
@@ -175,6 +195,7 @@ class Module:
             callback = chooser[3]
             payload = (chooser_obj, path_suffix, themes, callback)
             self.refresh_chooser(payload)
+        self.refreshing = False
 
     def refresh_chooser(self, payload):
         (chooser, path_suffix, themes, callback) = payload
@@ -183,15 +204,65 @@ class Module:
         if len(themes) > 0:
             inc = 1.0 / len(themes)
 
-        if path_suffix == "icons":
+        if path_suffix == 'icons':
+            cache_folder = GLib.get_user_cache_dir() + '/cs_themes/'
+            icon_cache_path = os.path.join(cache_folder, 'icons')
+
+            # Retrieve list of known themes/locations for faster loading (icon theme loading and lookup are very slow)
+            if os.path.exists(icon_cache_path):
+                read_path = icon_cache_path
+            else:
+                read_path = '/usr/share/cinnamon/cinnamon-settings/icons'
+
+            icon_paths = {}
+            with open(read_path, 'r') as cache_file:
+                for line in cache_file:
+                    theme_name, icon_path = line.strip().split(':')
+                    icon_paths[theme_name] = icon_path
+
+            dump = False
             for theme in themes:
-                icon_theme = Gtk.IconTheme()
-                icon_theme.set_custom_theme(theme)
-                folder = icon_theme.lookup_icon("folder", ICON_SIZE, Gtk.IconLookupFlags.FORCE_SVG)
-                if folder:
-                    path = folder.get_filename()
-                    chooser.add_picture(path, callback, title=theme, id=theme)
-                GLib.timeout_add(5, self.increment_progress, (chooser,inc))
+                theme_path = None
+
+                if theme in icon_paths:
+                    # loop through all possible locations until we find a match
+                    # (user folders should override system ones)
+                    for theme_folder in ICON_FOLDERS:
+                        possible_path = os.path.join(theme_folder, icon_paths[theme])
+                        if os.path.exists(possible_path):
+                            theme_path = possible_path
+                            break
+
+                if theme_path is None:
+                    icon_theme = Gtk.IconTheme()
+                    icon_theme.set_custom_theme(theme)
+                    folder = icon_theme.lookup_icon('folder', ICON_SIZE, Gtk.IconLookupFlags.FORCE_SVG)
+                    if folder:
+                        theme_path = folder.get_filename()
+
+                        # we need to get the relative path for storage
+                        for theme_folder in ICON_FOLDERS:
+                            if os.path.commonpath([theme_folder, theme_path]) == theme_folder:
+                                icon_paths[theme] = os.path.relpath(theme_path, start=theme_folder)
+                                break
+
+                    dump = True
+
+                if theme_path == None:
+                    continue;
+
+                if os.path.exists(theme_path):
+                    chooser.add_picture(theme_path, callback, title=theme, id=theme)
+                GLib.timeout_add(5, self.increment_progress, (chooser, inc))
+
+            if dump:
+                if not os.path.exists(cache_folder):
+                    os.mkdir(cache_folder)
+
+                with open(icon_cache_path, 'w') as cache_file:
+                    for theme_name, icon_path in icon_paths.items():
+                        cache_file.write('%s:%s\n' % (theme_name, icon_path))
+
         else:
             if path_suffix == "cinnamon":
                 chooser.add_picture("/usr/share/cinnamon/theme/thumbnail.png", callback, title="cinnamon", id="cinnamon")
@@ -270,15 +341,6 @@ class Module:
             print(detail)
         return True
 
-    def _on_metacity_theme_selected(self, path, theme):
-        try:
-            self.wm_settings.set_string("theme", theme)
-            self.metacity_chooser.set_button_label(theme)
-            self.metacity_chooser.set_tooltip_text(theme)
-        except Exception as detail:
-            print(detail)
-        return True
-
     def _on_gtk_theme_selected(self, path, theme):
         try:
             self.settings.set_string("gtk-theme", theme)
@@ -339,6 +401,8 @@ class Module:
         walked = walk_directories(dirs, lambda d: os.path.isdir(d), return_directories=True)
         valid = []
         for directory in walked:
+            if directory[0] in ("gnome", "hicolor"):
+                continue
             path = os.path.join(directory[1], directory[0], "index.theme")
             if os.path.exists(path):
                 try:
@@ -376,21 +440,6 @@ class Module:
             res.append((i[0], i[1]))
         return res
 
-    def _load_metacity_themes(self):
-        dirs = THEME_FOLDERS
-        valid = walk_directories(dirs, lambda d: os.path.exists(os.path.join(d, "metacity-1/metacity-theme-3.xml")), return_directories=True)
-        valid.sort(key=lambda a: a[0].lower())
-        res = []
-        for i in valid:
-            for j in res:
-                if i[0] == j[0]:
-                    if i[1] == dirs[0]:
-                        continue
-                    else:
-                        res.remove(j)
-            res.append((i[0], i[1]))
-        return res
-
     def _load_cinnamon_themes(self):
         dirs = THEME_FOLDERS
         valid = walk_directories(dirs, lambda d: os.path.exists(os.path.join(d, "cinnamon")), return_directories=True)
