@@ -5,6 +5,7 @@
  * @short_description: The file responsible for managing Cinnamon chrome
  */
 const Clutter = imports.gi.Clutter;
+const Cinnamon = imports.gi.Cinnamon;
 const GLib = imports.gi.GLib;
 const Gio = imports.gi.Gio;
 const Lang = imports.lang;
@@ -19,8 +20,7 @@ const EdgeFlip = imports.ui.edgeFlip;
 const HotCorner = imports.ui.hotCorner;
 const DeskletManager = imports.ui.deskletManager;
 const Panel = imports.ui.panel;
-
-const STARTUP_ANIMATION_TIME = 0.5;
+const StartupAnimation = imports.ui.startupAnimation;
 
 function isPopupMetaWindow(actor) {
     switch(actor.meta_window.get_window_type()) {
@@ -108,7 +108,7 @@ LayoutManager.prototype = {
     // not exist yet when the LayoutManager was constructed.
     init: function() {
         this._chrome.init();
-        
+
         this.edgeRight = new EdgeFlip.EdgeFlipper(St.Side.RIGHT, Main.wm.actionFlipWorkspaceRight);
         this.edgeLeft = new EdgeFlip.EdgeFlipper(St.Side.LEFT, Main.wm.actionFlipWorkspaceLeft);
 
@@ -119,7 +119,7 @@ LayoutManager.prototype = {
 
         this.hotCornerManager = new HotCorner.HotCornerManager();
     },
-    
+
     _toggleExpo: function() {
         if (Main.expo.animationInProgress)
             return;
@@ -130,7 +130,7 @@ LayoutManager.prototype = {
         }
         Main.expo.toggle();
     },
-    
+
     _updateMonitors: function() {
         let screen = global.screen;
 
@@ -139,27 +139,13 @@ LayoutManager.prototype = {
         for (let i = 0; i < nMonitors; i++)
             this.monitors.push(new Monitor(i, screen.get_monitor_geometry(i)));
 
-        if (nMonitors == 1) {
-            this.primaryIndex = this.bottomIndex = 0;
-        } else {
-            // If there are monitors below the primary, then we need
-            // to split primary from bottom.
-            this.primaryIndex = this.bottomIndex = screen.get_primary_monitor();
-            for (let i = 0; i < this.monitors.length; i++) {
-                let monitor = this.monitors[i];
-                if (this._isAboveOrBelowPrimary(monitor)) {
-                    if (monitor.y > this.monitors[this.bottomIndex].y)
-                        this.bottomIndex = i;
-                }
-            }
-        }
+        this.primaryIndex = screen.get_primary_monitor();
         this.primaryMonitor = this.monitors[this.primaryIndex];
-        this.bottomMonitor = this.monitors[this.bottomIndex];
     },
 
     _updateBoxes: function() {
         if (this.hotCornerManager)
-            this.hotCornerManager.updatePosition(this.primaryMonitor, this.bottomMonitor);
+            this.hotCornerManager.update();
         this._chrome._queueUpdateRegions();
     },
 
@@ -169,20 +155,6 @@ LayoutManager.prototype = {
         this.emit('monitors-changed');
     },
 
-    _isAboveOrBelowPrimary: function(monitor) {
-        let primary = this.monitors[this.primaryIndex];
-        let monitorLeft = monitor.x, monitorRight = monitor.x + monitor.width;
-        let primaryLeft = primary.x, primaryRight = primary.x + primary.width;
-
-        if ((monitorLeft >= primaryLeft && monitorLeft < primaryRight) ||
-            (monitorRight > primaryLeft && monitorRight <= primaryRight) ||
-            (primaryLeft >= monitorLeft && primaryLeft < monitorRight) ||
-            (primaryRight > monitorLeft && primaryRight <= monitorRight))
-            return true;
-
-        return false;
-    },
-
     get focusIndex() {
         let i = 0;
         if (global.stage.key_focus != null)
@@ -216,44 +188,30 @@ LayoutManager.prototype = {
 
         this.keyboardBox.hide();
 
-        let monitor = this.primaryMonitor;
-        let x = monitor.x + monitor.width / 2.0;
-        let y = monitor.y + monitor.height / 2.0;
-
-        Main.uiGroup.set_pivot_point(x / global.screen_width,
-                                     y / global.screen_height);
-        Main.uiGroup.scale_x = Main.uiGroup.scale_y = 0.75;
-        Main.uiGroup.opacity = 0;
+        global.stage.hide_cursor();
         global.background_actor.show();
-        global.window_group.set_clip(monitor.x, monitor.y, monitor.width, monitor.height);
+
+        this.startupAnimation = new StartupAnimation.Animation(this.primaryMonitor,
+                                                               ()=>this._startupAnimationComplete());
+        this._chrome.updateRegions();
     },
 
-    _startupAnimation: function() {
+    _doStartupAnimation: function() {
         // Don't animate the strut
         this._chrome.freezeUpdateRegions();
-        Tweener.addTween(Main.uiGroup,
-                         { scale_x: 1,
-                           scale_y: 1,
-                           opacity: 255,
-                           time: STARTUP_ANIMATION_TIME,
-                           transition: 'easeOutQuad',
-                           onComplete: this._startupAnimationComplete,
-                           onCompleteScope: this });       
+        this.startupAnimation.run();
     },
 
     _startupAnimationComplete: function() {
-        global.stage.no_clear_hint = true;
-        this._coverPane.destroy();
+        global.stage.show_cursor();
+        this.removeChrome(this._coverPane);
         this._coverPane = null;
-
-        global.window_group.remove_clip();
         this._chrome.thawUpdateRegions();
 
         Main.setRunState(Main.RunState.RUNNING);
     },
 
     showKeyboard: function () {
-        if (Main.messageTray) Main.messageTray.hide();
         if (this.hideIdleId > 0) {
             Mainloop.source_remove(this.hideIdleId);
             this.hideIdleId = 0;
@@ -294,8 +252,6 @@ LayoutManager.prototype = {
     },
 
     hideKeyboard: function (immediate) {
-        if (Main.messageTray) Main.messageTray.hide();
-
         this.keyboardBox.hide();
         this._chrome.modifyActorParams(this.keyboardBox, { affectsStruts: false });
         this._chrome.updateRegions();
@@ -304,7 +260,7 @@ LayoutManager.prototype = {
         return false;
     },
 
-    /** 
+    /**
      * updateChrome:
      * @doVisibility (boolean): (optional) whether to recalculate visibility.
      *
@@ -526,8 +482,8 @@ Chrome.prototype = {
             return;
         let actorData = this._trackedActors[i];
 
-        if (actorData.addToWindowgroup) global.window_group.remove_actor(actor);
-        else Main.uiGroup.remove_actor(actor);
+        if (actorData.addToWindowgroup) global.window_group.remove_child(actor);
+        else Main.uiGroup.remove_child(actor);
         this._untrackActor(actor);
     },
 
@@ -539,7 +495,7 @@ Chrome.prototype = {
         }
         return -1;
     },
-    
+
     modifyActorParams: function(actor, params) {
         let index = this._findActor(actor);
         if (index == -1)
@@ -606,7 +562,21 @@ Chrome.prototype = {
             let actorData = this._trackedActors[i], visible;
             if (!actorData.isToplevel)
                 continue;
-            else if (this._inOverview)
+            else if (global.stage_input_mode == Cinnamon.StageInputMode.FULLSCREEN) {
+                let monitor = this.findMonitorForActor(actorData.actor);
+
+                if (global.screen.get_n_monitors() == 1 || !monitor.inFullscreen) {
+                    visible = true;
+                } else {
+                    if (Main.modalActorFocusStack.length > 0) {
+                        let modalActor = Main.modalActorFocusStack[Main.modalActorFocusStack.length - 1].actor;
+
+                        if (this.findMonitorForActor(modalActor) == monitor) {
+                            visible = true;
+                        }
+                    }
+                }
+            } else if (this._inOverview)
                 visible = true;
             else if (!actorData.visibleInFullscreen &&
                      this.findMonitorForActor(actorData.actor).inFullscreen)
@@ -707,7 +677,14 @@ Chrome.prototype = {
     },
 
     _windowsRestacked: function() {
-        if (this._isPopupWindowVisible != global.top_window_group.get_children().some(isPopupMetaWindow))
+        // Figure out where the pointer is in case we lost track of
+        // it during a grab.
+        global.sync_pointer();
+
+        let isPopupWindowVisible = global.top_window_group.get_children().some(isPopupMetaWindow);
+        let popupVisibilityChanged = this._isPopupWindowVisible !== isPopupWindowVisible;
+        this._isPopupWindowVisible = isPopupWindowVisible;
+        if (popupVisibilityChanged)
             this._updateVisibility();
         else
             this._queueUpdateRegions();
@@ -721,8 +698,7 @@ Chrome.prototype = {
             this._updateRegionIdle = 0;
         }
 
-        let isPopupMenuVisible = global.top_window_group.get_children().some(isPopupMetaWindow);
-        let wantsInputRegion = !isPopupMenuVisible;
+        let wantsInputRegion = !this._isPopupWindowVisible;
 
         for (let i = 0; i < this._trackedActors.length; i++) {
             let actorData = this._trackedActors[i];
@@ -816,7 +792,6 @@ Chrome.prototype = {
         }
 
         global.set_stage_input_region(rects);
-        this._isPopupWindowVisible = isPopupMenuVisible;
 
         let screen = global.screen;
         for (let w = 0; w < screen.n_workspaces; w++) {
